// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'

var appVersion = "0.0.0";

angular.module('starter', [
  'ionic',
  'ngCordova',
  'ngOpenFB',
  'ngCordovaBeacon',
  'angularMoment',
  'angular.filter',
  'LocalStorageModule',
  'techParade.ctrl'
  ])

.run(function($ionicPlatform, $rootScope, $state, $ionicLoading, $ionicModal, $cordovaBeacon, $timeout, $interval, $http, $window, $ionicPopup, ngFB, localStorageService, moment, $cordovaPushV5, $ionicScrollDelegate, $cordovaGoogleAnalytics, $cordovaLocalNotification, $location) {
  $ionicPlatform.ready(function() {

    if(window.StatusBar) {
      StatusBar.styleDefault();
    }

    if (window.cordova) {
      cordova.getAppVersion.getVersionNumber().then(function(version) {
         appVersion = version;
      });
      if (window.cordova.plugins.Keyboard) {
        cordova.plugins.Keyboard.disableScroll(true);
      }
    }

    var options = {
      android: {
        senderID: "287721458015"
      },
      ios: {
        alert: "true",
        badge: "true",
        sound: "true"
      }
    };

    document.addEventListener("deviceready", function() {
      $cordovaGoogleAnalytics.debugMode();
      $cordovaGoogleAnalytics.startTrackerWithId('UA-82287325-1');
    });

    // initialize
    $cordovaPushV5.initialize(options).then(function() {
      $cordovaPushV5.onNotification();    // start listening for new notifications
      $cordovaPushV5.onError();           // start listening for errors
      $cordovaPushV5.register().then(function(data) {   // register to get registrationId
        // sauvegarde du token en local pour non register à nouveau ou suppression
        // localStorageService.set('push_token', data);

        // sauvegarde du token sur notre serveur
        $http.post('http://technoparade.trgs.at/api/push_notification/register', {
            registration_id: data,
            app_version: appVersion,
            platform: ionic.Platform.platform()
          }).then(function(response) {
          console.log('--- REGISTRATION Trgs: [OK]');
          console.log(response);
        });
      })
    });

    // triggered every time notification received
    $rootScope.$on('$cordovaPushV5:notificationReceived', function(event, data) {
      data.message = data.message.replace(/(?:\r\n|\r|\n)/g, '<br />');
      $ionicPopup.alert({
        title: (typeof data.title !== 'undefined' ? data.title : 'Techno Parade'),
        template: data.message
      });

      var id_notification = Math.round((Math.pow(36, 10 + 1) - Math.random() * Math.pow(36, 10))).toString(36).slice(1);

      // suppression du badge sur l'icône de l'app
      $cordovaPushV5.setBadgeNumber(0);

      // on enregistre le passage sur ce beacon pour ne pas le dupliquer
      $rootScope.events = localStorageService.get('events') || {};
      $rootScope.events[id_notification] = {
        date: moment().toDate(),
        title: (typeof data.title !== 'undefined' ? data.title : 'Techno Parade'),
        text: data.message
      };
      localStorageService.set('events', $rootScope.events);

      // data.count,
      // data.sound,
      // data.image,
      // data.additionalData
    });

    // triggered every time error occurs
    $rootScope.$on('$cordovaPushV5:errorOcurred', function(event, e){
      console.log('error message: ' + e.message)
    });
  });

  // scroll en haut de la page à chaque changement de page.
  $rootScope.$on("$stateChangeSuccess", function() {
    $ionicScrollDelegate.scrollTop();
  });

  // initialiser la connexion facebook
  ngFB.init({appId: '155979051500295'});

  $rootScope.goToPage = function(state, params, tracking) {
    if (typeof tracking !== 'undefined') {
      console.log(tracking);
      document.addEventListener("deviceready", function() {
        $cordovaGoogleAnalytics.trackEvent(tracking.category, tracking.event, tracking.label);
      });
    }
    $state.go(state, params);
  };

  $rootScope.openExternal = function(url, tracking) {
    if (url != '') {
      if (typeof tracking !== 'undefined') {
        console.log(tracking);
        document.addEventListener("deviceready", function() {
          $cordovaGoogleAnalytics.trackEvent(tracking.category, tracking.event, tracking.label);
        });
      }
      $window.open(url, "_system");
    }
  };

  // display the loading overlay when receiving the intermediary event
  $rootScope.$on('loading:show', function() {
    $ionicLoading.show({template: '<ion-spinner icon="lines" class="spinner-light"></ion-spinner>'});
  });
  $rootScope.$on('loading:hide', function() {
    $ionicLoading.hide();
  });

  // page modal du menu
  $ionicModal.fromTemplateUrl('menu.html', {
    scope: $rootScope,
    animation: 'slide-in-up',
    hardwareBackButtonClose: true
  }).then(function(modal) {
    $rootScope.menuModal = modal;
  });
  $rootScope.closeMenu = function() { $rootScope.menuModal.hide(); };
  $rootScope.openMenu = function() { $rootScope.menuModal.show(); };

  // page modal des notifs
  $ionicModal.fromTemplateUrl('notifs.html', {
    scope: $rootScope,
    animation: 'slide-in-down'
  }).then(function(modal) {
    $rootScope.notifModal = modal;
  });
  $rootScope.closeNotifs = function() {$rootScope.notifModal.hide();};
  $rootScope.openNotifs = function() {
    $rootScope.notifModal.show();
    document.addEventListener("deviceready", function() {
      $cordovaGoogleAnalytics.trackEvent('header', 'mes_notifications', 'header_picto_notifications');
    });
  };

  // nettoyage de la mémoire
  $rootScope.$on('$destroy', function() {
    $rootScope.menuModal.remove();
    $rootScope.notifModal.remove();
    $interval.cancel(stop);
  });

  // récupération de la config de l'app
  $http.get('http://technoparade.trgs.at/config.json?id=' + moment().unix()).then(function(response) {
    if (response.status == 200 && response.data != null) {
      $rootScope.config = response.data;
      console.log(JSON.stringify(response.data))
      // Recuperer les dates d'aujourd'hui & de la technoparade

      if(moment().isSame($rootScope.config.date_event, 'day') ){
        $rootScope.dday = true;
      }

      // par défaut, on télécharge les nouvelle données
      var downloadData = true;
      console.log(localStorageService.keys().indexOf('config'))
      if (localStorageService.keys().indexOf('config') != -1) {
       
        var oldConfig = localStorageService.get('config');
        var savedVersion = moment(oldConfig.date_update);
        var currentVersion = moment($rootScope.config.date_update);
        // si la version actuelle est la même (ou avant) que la version enregistrée, on ne télécharge pas les données
        downloadData = currentVersion.isAfter(savedVersion);
        console.log(downloadData);
      }

      // on enregistre la config de l'app pour avoir la date de dernière mise à jour
      localStorageService.set('config', $rootScope.config);

      // affichage de l'intersticiel
      if ($rootScope.config.interstitiel != null) {
        var templateInterciel = '<img src="' + $rootScope.config.interstitiel.img + '" >';
        if ($rootScope.config.interstitiel.url != '') {
          templateInterciel = '<a target="_system" href="' + $rootScope.config.interstitiel.url + '">' + templateInterciel + '</a>';
        }

        $ionicPopup.alert({
          title: $rootScope.config.interstitiel.titre,
          template: templateInterciel
        });
      }

      // calcul du nombre de jours restants avant l'event
      var today = moment();
      var dateEvent = moment($rootScope.config.date_event);
      var diff = today.diff(dateEvent, 'days');

      if (diff < 0) {

        var countdown = $ionicPopup.alert({
          title: 'Compte à rebours',
          cssClass: 'compte-a-rebours',
          template: ' <button class="close button button-icon icon ion-close-round" ng-click="closeCompte()"></button> <p class="j">J' + diff + '</p>',
          buttons: {}
        })
        $timeout(function() {
          countdown.close();
        }, 10000);

        $rootScope.closeCompte = function(){
          countdown.close();
        }

      }

      if (downloadData == true || localStorageService.keys().indexOf('data') == -1) {
        // vidage du cache des beacons au cas où les messages des organisations ont été mis à jour
        localStorageService.remove('events');
        // récupération des contenus de l'app. all_dev.json
        $http.get('http://technoparade.trgs.at/all.json?id=' + moment().unix()).then(function(response) {
          if (response.status == 200 && response.data != null) {
            $rootScope.data = response.data;
            var event = new Event('onDataLoaded');
            document.dispatchEvent(event);
            console.log('data loaded', response.data)

            // Set la limit pour le truncate du text
            var n = $rootScope.data.edito.text.indexOf('<!--break-->');
            $rootScope.data.edito.truncateIndex = n;
            $rootScope.data.edito.limit = $rootScope.data.edito.truncateIndex;

            for (var i = $rootScope.data.after.liste.length - 1; i >= 0; i--) {
              $rootScope.data.after.liste[i].date = moment($rootScope.data.after.liste[i].date).toDate();
            }

            // parser les ordres des partenaires en int
            for (var property in $rootScope.data.partenaires) {
              if ($rootScope.data.partenaires.hasOwnProperty(property)) {
                for (var i = $rootScope.data.partenaires[property].length - 1; i >= 0; i--) {
                  $rootScope.data.partenaires[property][i].order = parseInt($rootScope.data.partenaires[property][i].order);
                }
              }
            }

            // on enregistre les contenus pour ne pas les télécharger plusieurs fois
            localStorageService.set('data', $rootScope.data);
            setupBeacons();
          } else {
            $ionicPopup.alert({
              title: 'Erreur',
              template: 'Impossible de récupérer le contenu de l\'application'
            });
          }
        });
      } else {
        // on récupère les données déjà enregistrées dans le localstorage
        $rootScope.data = localStorageService.get('data');
        setupBeacons();
      }
    } else {
      $ionicPopup.alert({
        title: 'Erreur',
        template: 'Impossible de récupérer le contenu de l\'application'
      });
    }
  });

  var setupBeacons = function() {
    $rootScope.beacons = [];
    $rootScope.events = localStorageService.get('events') || {};

    var checkBeaconChar = function() {
      // pour chaque beacons à proximité
      for (var i = 0; i < $rootScope.beacons.length; i++) {
        // vérifier si les major/minor de ce beacon correspondent à un char précis
        for (var idx in $rootScope.data.chars) {
          // si un beacon est lié à ce char
          // && que ce beacon est à proximité
          // && qu'on n'a pas déjà enregistré ce beacon
          if (typeof $rootScope.data.chars[idx].beacon === 'object'
            && $rootScope.data.chars[idx].beacon !== null
            && $rootScope.data.chars[idx].beacon.length > 0) {

            for (var j = 0; j < $rootScope.data.chars[idx].beacon.length; j++) {
              if ($rootScope.data.chars[idx].beacon[j].hasOwnProperty('major')
                && $rootScope.beacons[i].major == $rootScope.data.chars[idx].beacon[j].major
                && $rootScope.beacons[i].minor == $rootScope.data.chars[idx].beacon[j].minor
                && $rootScope.data.chars[idx].beacon[j].text != null
                && !$rootScope.events.hasOwnProperty($rootScope.data.chars[idx].beacon[j].id_char)) {

                // on enregistre le passage sur ce beacon pour ne pas le dupliquer
                $rootScope.events[$rootScope.data.chars[idx].beacon[j].id_char] = {
                  date: moment().toDate(),
                  major: $rootScope.beacons[i].major,
                  minor: $rootScope.beacons[i].minor,
                  title: $rootScope.data.chars[idx].beacon[j].titre,
                  text: $rootScope.data.chars[idx].beacon[j].text
                };
                localStorageService.set('events', $rootScope.events);

                // on crée une notif avec le texte du char
                $cordovaLocalNotification.schedule({
                  id: idx,
                  title: $rootScope.data.chars[idx].beacon[j].titre,
                  text: $rootScope.data.chars[idx].beacon[j].text
                });

                // on popup dans l'app
                $ionicPopup.alert({
                  title: $rootScope.data.chars[idx].beacon[j].titre,
                  template: $rootScope.data.chars[idx].beacon[j].text
                });
              }
            }
          }
        }
      }
    };

    document.addEventListener('deviceready', function() {
      // les beacons ont les même identifiants (différence sur major/minor)
      var beaconIdentifier = 'Kontakt';
      var beaconUuid = 'f7826da6-4fa2-4e98-8024-bc5b71e0893e';
      var beaconRegion = $cordovaBeacon.createBeaconRegion(beaconIdentifier, beaconUuid);

      // on demande d'activer le bluetooth s'il ne l'est pas déjà
      $cordovaBeacon.requestAlwaysAuthorization();
      $cordovaBeacon.enableBluetooth();

      // calcul du nombre de jours restants avant l'event
      var today = moment();
      var dateEvent = moment($rootScope.config.date_event);
      var diff = today.diff(dateEvent, 'days');

      if (diff == 0) {
        $cordovaBeacon.isBluetoothEnabled().then(function(response) {
          if (!response) {
            $ionicPopup.alert({
              title: 'Bluetooth',
              template: $rootScope.config.beacon_str
            });
          }
        });
      }

      // callback lorsqu'on détecte des beacons aux alentours
      $rootScope.$on('$cordovaBeacon:didRangeBeaconsInRegion', function(event, pluginResult) {
        // on enregistre la liste des beacons détectés
        $rootScope.beacons = [];
        for (var i = 0; i < pluginResult.beacons.length; i++) {
          $rootScope.beacons.push({major: pluginResult.beacons[i].major, minor: pluginResult.beacons[i].minor});
        }
        $rootScope.$apply();
      });

      // on rafraichit la liste à intervals réguliers
      $cordovaBeacon.startRangingBeaconsInRegion(beaconRegion);
      $interval(function() {
        // $timeout(function() {
          // $cordovaBeacon.stopRangingBeaconsInRegion(beaconRegion);
          checkBeaconChar();
        // }, 5000);
      }, 5000);

      $rootScope.$on('$cordovaLocalNotification:click', function(event, notification, state) {
        $ionicPopup.alert({
          title: notification.title,
          template: notification.text
        });
      });
    });
  };
})

.config(function($ionicConfigProvider, $stateProvider, $urlRouterProvider, $httpProvider, localStorageServiceProvider) {
  // normaliser la configuration sur android/ios
  $ionicConfigProvider.backButton.icon('ion-ios-arrow-thin-left');
  $ionicConfigProvider.backButton.text('');
  $ionicConfigProvider.backButton.previousTitleText(false);
  $ionicConfigProvider.navBar.alignTitle('center');
  $ionicConfigProvider.tabs.position('top');
  $ionicConfigProvider.tabs.style('striped');

  localStorageServiceProvider.setPrefix('techParade');

  $stateProvider
    .state('home', {
      url: '/home',
      templateUrl: 'templates/home.html',
      controller: 'HomeCtrl'
    })
    .state('parcours', {
      url: '/parcours',
      templateUrl: 'templates/parcours.html',
      controller: 'ParcoursCtrl'
    })
    .state('chars', {
      url: '/chars',
      templateUrl: 'templates/chars/liste.html',
      controller: function($scope, $cordovaGoogleAnalytics) {
        $scope.$on('$stateChangeSuccess', function() {
          document.addEventListener("deviceready", function() {
            $cordovaGoogleAnalytics.trackView('/chars');
          });
        });
      }
    })
    .state('chars-detail', {
      url: '/chars/{idChar}',
      templateUrl: 'templates/chars/detail.html',
      controller: 'CharsDetailCtrl'
    })
    .state('chars-artiste', {
      url: '/chars/{idChar}/artiste/{idArtiste}',
      templateUrl: 'templates/artiste.html',
      controller: 'CharsArtisteCtrl'
    })
    .state('after', {
      url: '/after',
      templateUrl: 'templates/after/liste.html',
      controller: function($scope, $cordovaGoogleAnalytics) {
        $scope.$on('$stateChangeSuccess', function() {
          document.addEventListener("deviceready", function() {
            $cordovaGoogleAnalytics.trackView('/after');
          });
        });
      }
    })
    .state('after-detail', {
      url: '/after/{idAfter}',
      templateUrl: 'templates/after/detail.html',
      controller: 'AfterDetailCtrl'
    })
    .state('after-artiste', {
      url: '/after/{idAfter}/artiste/{idArtiste}',
      templateUrl: 'templates/artiste.html',
      controller: 'AfterArtisteCtrl'
    })
    .state('anges', {
      url: '/anges',
      templateUrl: 'templates/anges.html',
      controller: function($scope, $cordovaGoogleAnalytics) {
        $scope.$on('$stateChangeSuccess', function() {
          document.addEventListener("deviceready", function() {
            $cordovaGoogleAnalytics.trackView('/anges');
          });
        });
      }
    })
    .state('infos', {
      url: '/infos',
      templateUrl: 'templates/infos/home.html',
      controller: function($scope, $cordovaGoogleAnalytics) {
        $scope.$on('$stateChangeSuccess', function() {
          document.addEventListener("deviceready", function() {
            $cordovaGoogleAnalytics.trackView('/infos');
          });
        });
      }
    })
    .state('infos-benevole', {
      url: '/infos/benevole',
      templateUrl: 'templates/infos/benevole.html',
      controller: function($scope, $cordovaGoogleAnalytics) {
        $scope.$on('$stateChangeSuccess', function() {
          document.addEventListener("deviceready", function() {
            $cordovaGoogleAnalytics.trackView('/infos/benevole');
          });
        });
        // afficher l'icone de menu si on n'a perdu l'historique
        $scope.$on('$ionicView.beforeEnter', function(e, data) {
          $scope.showMenuIcon = !data.enableBack;
        });
      }
    })
    .state('infos-sante', {
      url: '/infos/sante',
      templateUrl: 'templates/infos/sante.html',
      controller: function($scope, $cordovaGoogleAnalytics) {
        $scope.$on('$stateChangeSuccess', function() {
          document.addEventListener("deviceready", function() {
            $cordovaGoogleAnalytics.trackView('/infos/sante');
          });
        });
        // afficher l'icone de menu si on n'a perdu l'historique
        $scope.$on('$ionicView.beforeEnter', function(e, data) {
          $scope.showMenuIcon = !data.enableBack;
        });
      }
    })
    .state('infos-credits', {
      url: '/infos/credits',
      templateUrl: 'templates/infos/credits.html',
      controller: function($scope, $cordovaGoogleAnalytics) {
        $scope.$on('$stateChangeSuccess', function() {
          document.addEventListener("deviceready", function() {
            $cordovaGoogleAnalytics.trackView('/infos/credits');
          });
        });
        // afficher l'icone de menu si on n'a perdu l'historique
        $scope.$on('$ionicView.beforeEnter', function(e, data) {
          $scope.showMenuIcon = !data.enableBack;
        });
      }
    })
    .state('partenaires', {
      url: '/partenaires',
      templateUrl: 'templates/partenaires.html',
      controller: function($scope, $cordovaGoogleAnalytics) {
        $scope.$on('$stateChangeSuccess', function() {
          document.addEventListener("deviceready", function() {
            $cordovaGoogleAnalytics.trackView('/partenaires');
          });
        });
      }
    })
    .state('social', {
      url: '/social',
      templateUrl: 'templates/social.html',
      controller: 'SocialCtrl'
    });

  // lorsqu'on arrive sur l'app
  $urlRouterProvider.otherwise('/home');

  // intercept http requests to show a loading overlay
  $httpProvider.interceptors.push(function($rootScope) {
    return {
      request: function(config) {
        // only intercept when we want to
        if (config.intercept !== false) {
          $rootScope.$broadcast('loading:show');
        }
        return config;
      },
      response: function(response) {
        $rootScope.$broadcast('loading:hide');
        return response;
      },
      responseError: function(response) {
        if (response.config.intercept !== false) {
          $rootScope.connexionServeur = false;
        }
        $rootScope.$broadcast('loading:hide');
        return response;
      }
    };
  });

  // use x-www-form-urlencoded Content-Type
  $httpProvider.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded;charset=utf-8';

  // function that will convert json encoded to url encoded params for post
  var param = function(obj) {
    var query = '', name, value, fullSubName, subName, subValue, innerObj, i;

    for (name in obj) {
      value = obj[name];

      if (value instanceof Array) {
        for (i = 0; i < value.length; ++i) {
          subValue = value[i];
          fullSubName = name + '[' + i + ']';
          innerObj = {};
          innerObj[fullSubName] = subValue;
          query += param(innerObj) + '&';
        }
      }
      else if (value instanceof Object) {
        for (subName in value) {
          subValue = value[subName];
          fullSubName = name + '[' + subName + ']';
          innerObj = {};
          innerObj[fullSubName] = subValue;
          query += param(innerObj) + '&';
        }
      }
      else if (value !== undefined && value !== null) {
        query += encodeURIComponent(name) + '=' + encodeURIComponent(value) + '&';
      }
    }

    return query.length ? query.substr(0, query.length - 1) : query;
  };

  // override $http service's default transformRequest to read $_POST directly in PHP on the server side
  $httpProvider.defaults.transformRequest = [function(data) {
    return angular.isObject(data) && String(data) !== '[object File]' ? param(data) : data;
  }];
})

.directive('tpMenuClose', ['$ionicHistory', '$timeout', function($ionicHistory, $timeout) {
  return {
    restrict: 'AC',
    link: function($scope, $element) {
      $element.bind('click', function() {
        $ionicHistory.nextViewOptions({
          historyRoot: true,
          disableAnimate: true,
          expire: 300
        });
        // if no transition in 300ms, reset nextViewOptions
        // the expire should take care of it, but will be cancelled in some
        // cases. This directive is an exception to the rules of history.js
        $timeout( function() {
          $ionicHistory.nextViewOptions({
            historyRoot: false,
            disableAnimate: false
          });
        }, 300);
      });
    }
  };
}]);