angular.module('techParade.ctrl', [])

.controller('HomeCtrl', function($scope, $rootScope, $cordovaGoogleAnalytics, $ionicScrollDelegate, $cordovaGeolocation, $ionicPopup, $http) {
  $scope.$on('$stateChangeSuccess', function() {
    document.addEventListener("deviceready", function() {
      $cordovaGoogleAnalytics.trackView('/home');
    });

    $scope.updateMap = function() {
      $cordovaGoogleAnalytics.trackEvent('parcours', 'geolocaliser_la_parade', 'parcours_geolocaliser_la_parade');

      $http.get('http://technoparade.trgs.at/api/geolocation').then(function(response) {
        if (response.status == 200 && response.data != null) {
          var pos = [Number(response.data.geoloc.longitude), Number(response.data.geoloc.latitude)];
          if ($rootScope.tp_marker == null)
            $rootScope.tp_marker = $scope.map.addMarker('img/logos/technoparade_marker.png', pos);
          else
            $rootScope.tp_marker.setPosition(ol.proj.fromLonLat(pos));
          $scope.data.parcours.geoloc = response.data.geoloc;
        } else {
          $ionicPopup.alert({
            title: 'Erreur',
            template: 'Impossible de récupérer la position de la parade'
          });
        }
      });
    }

    $scope.onDataLoaded = function() {
      $scope.map = new MapService();
      $scope.map.setGpxData($rootScope.data.parcours.etapes_gps_openstreetmap);
      $rootScope.tp_marker = null;

      $cordovaGeolocation.getCurrentPosition({timeout: 10000, enableHighAccuracy: true}).then(function(position) {
        console.log(position);
        $scope.map.drawMap('maphome', [position.coords.longitude, position.coords.latitude]);
        let start = $rootScope.data.parcours.etapes_gps[0].split(';')
        $scope.map.addMarker('img/logos/start_marker.png', [Number(start[1]), Number(start[0])]);
        $scope.map.addMarker('img/logos/marker.png', [position.coords.longitude, position.coords.latitude]);
        start = $rootScope.data.parcours.etapes_gps[1].split(';')
        $scope.map.addMarker('img/logos/end_marker.png', [Number(start[1]), Number(start[0])]);
          $scope.map.map.once('postrender', function(event) {
          $scope.updateMap();
        });
      }, function(error) {
        console.error(error)
        $ionicPopup.alert({
          title: 'Géolocalisation',
          template: 'Activez votre géolocation pour afficher votre position sur la carte.'
        });
      });
    }
    if ($rootScope.dday) {
      if (!$rootScope.data)
        document.addEventListener('onDataLoaded', $scope.onDataLoaded, false);
      else {
        $scope.onDataLoaded();
      }
    }
  });

  $scope.expandText = function() {
    $rootScope.data.edito.limit = $rootScope.data.edito.text.length;
    $ionicScrollDelegate.resize();
    document.addEventListener("deviceready", function() {
      $cordovaGoogleAnalytics.trackEvent('home', 'edito_lire_la_suite', 'home_lire_la_suite');
    });
  };

  $scope.reduceText = function() {
    $rootScope.data.edito.limit = $rootScope.data.edito.truncateIndex;
    $ionicScrollDelegate.resize();
    document.addEventListener("deviceready", function() {
      $cordovaGoogleAnalytics.trackEvent('home', 'edito_reduire_le_texte', 'home_edito_reduire_le_texte');
    });
  };

  $scope.$on("$stateChangeSuccess", function() {
    if ($rootScope.hasOwnProperty('data')) {
      $rootScope.data.edito.limit = $rootScope.data.edito.truncateIndex;
      for (var char in $rootScope.data.chars) {
        if ($rootScope.data.chars.hasOwnProperty(char)) {
          $rootScope.data.chars[char].random = Math.random();
        }
      }
    }
  });
})

.controller('ParcoursCtrl', function($scope, $rootScope, $state, $cordovaGeolocation, $http, $ionicPopup, $cordovaGoogleAnalytics) {
    $scope.$on('$stateChangeSuccess', function() {
      document.addEventListener("deviceready", function() {
        $cordovaGoogleAnalytics.trackView('/parcours');
      });
    });

    $scope.map = new MapService();
    $scope.map.setGpxData($scope.data.parcours.etapes_gps_openstreetmap);
    $rootScope.tp_marker = null;

    $cordovaGeolocation.getCurrentPosition({timeout: 10000, enableHighAccuracy: true}).then(function(position) {
      $scope.map.drawMap('map', [position.coords.longitude, position.coords.latitude]);
      let start = $scope.data.parcours.etapes_gps[0].split(';')
      $scope.map.addMarker('img/logos/start_marker.png', [Number(start[1]), Number(start[0])]);
      $scope.map.addMarker('img/logos/marker.png', [position.coords.longitude, position.coords.latitude]);
      start = $scope.data.parcours.etapes_gps[1].split(';')
      $scope.map.addMarker('img/logos/end_marker.png', [Number(start[1]), Number(start[0])]);
        $scope.map.map.once('postrender', function(event) {
        $scope.updateMap();
      });
    }, function(error) {
      $ionicPopup.alert({
        title: 'Géolocalisation',
        template: 'Activez votre géolocation pour afficher votre position sur la carte.'
      });
    });

    $scope.updateMap = function() {
      document.addEventListener("deviceready", function() {
        $cordovaGoogleAnalytics.trackEvent('parcours', 'geolocaliser_la_parade', 'parcours_geolocaliser_la_parade');
      });

      $http.get('http://technoparade.trgs.at/api/geolocation').then(function(response) {
        if (response.status == 200 && response.data != null) {
          var pos = [Number(response.data.geoloc.longitude), Number(response.data.geoloc.latitude)];
          if ($rootScope.tp_marker == null)
            $rootScope.tp_marker = $scope.map.addMarker('img/logos/technoparade_marker.png', pos);
          else
            $rootScope.tp_marker.setPosition(ol.proj.fromLonLat(pos));
          $scope.data.parcours.geoloc = response.data.geoloc;
        } else {
          $ionicPopup.alert({
            title: 'Erreur',
            template: 'Impossible de récupérer la position de la parade'
          });
        }
      });
    }
})

.controller('CharsDetailCtrl', function($scope, $rootScope, $stateParams, $timeout) {
  if ($stateParams.hasOwnProperty('idChar')) {
    $scope.idChar = $stateParams.idChar;

    $timeout(function() {
      $scope.char = $rootScope.data.chars[$scope.idChar];
      document.addEventListener("deviceready", function() {
        $cordovaGoogleAnalytics.trackView('/chars/' + $scope.char.id);
      });
    }, 750);
  }

  // afficher l'icone de menu si on n'a perdu l'historique
  $scope.$on('$ionicView.beforeEnter', function(e, data) {
    $scope.showMenuIcon = !data.enableBack;
  });
})

.controller('CharsArtisteCtrl', function($scope, $rootScope, $stateParams, $timeout, $cordovaGoogleAnalytics, $http) {
  if ($stateParams.hasOwnProperty('idChar') && $stateParams.hasOwnProperty('idArtiste')) {
    $timeout(function() {
      $scope.title = $rootScope.data.chars[$stateParams.idChar].nom;
      $scope.artiste = $rootScope.data.chars[$stateParams.idChar].artistes[$stateParams.idArtiste];
      setTimeout(function() {
        if ($scope.artiste.player_soundcloud && $scope.artiste.player_soundcloud.length > 0)
          document.getElementById('soundcloud').src = 'https://w.soundcloud.com/player/?url=https%3A//api.soundcloud.com/tracks/' + $scope.artiste.player_soundcloud + '&color=%23ff5500&auto_play=false&hide_related=false&show_comments=true&show_user=true&show_reposts=false&show_teaser=true&visual=true';
        if ($scope.artiste.player_deezer && $scope.artiste.player_deezer.length > 0)
          document.getElementById('deezer').src = 'https://www.deezer.com/plugins/player?format=classic&autoplay=false&playlist=true&width=100%&height=166&color=007FEB&layout=dark&size=medium&type=tracks&app_id=1&id=' + $scope.artiste.player_deezer;
      }, 200);

      document.addEventListener("deviceready", function() {
        $cordovaGoogleAnalytics.trackView('/chars/' + $rootScope.data.chars[$stateParams.idChar].id + '/artiste/' + $scope.artiste.id);
        $cordovaGoogleAnalytics.trackEvent('page_char_' + $rootScope.data.chars[$stateParams.idChar].id, 'artiste_' + $scope.artiste.id, 'page_char_' + $rootScope.data.chars[$stateParams.idChar].id + '_artiste_' + $scope.artiste.id);
      });
    }, 750);
  }

  // afficher l'icone de menu si on n'a perdu l'historique
  $scope.$on('$ionicView.beforeEnter', function(e, data) {
    $scope.showMenuIcon = !data.enableBack;
  });
})

.controller('AfterDetailCtrl', function($scope, $rootScope, $stateParams, $timeout) {
  if ($stateParams.hasOwnProperty('idAfter')) {
    $scope.idAfter = $stateParams.idAfter;

    $timeout(function() {
      $scope.after = $rootScope.data.after.liste[$stateParams.idAfter];
      document.addEventListener("deviceready", function() {
        $cordovaGoogleAnalytics.trackView('/after/' + $scope.after.id);
      });
    }, 750);
  }

  // afficher l'icone de menu si on n'a perdu l'historique
  $scope.$on('$ionicView.beforeEnter', function(e, data) {
    $scope.showMenuIcon = !data.enableBack;
  });
})

.controller('AfterArtisteCtrl', function($scope, $rootScope, $stateParams, $timeout) {
  if ($stateParams.hasOwnProperty('idAfter') && $stateParams.hasOwnProperty('idArtiste')) {
    $timeout(function() {
      $scope.title = $rootScope.data.after.liste[$stateParams.idAfter].titre;
      $scope.artiste = $rootScope.data.after.liste[$stateParams.idAfter].artistes[$stateParams.idArtiste];

      document.addEventListener("deviceready", function() {
        $cordovaGoogleAnalytics.trackView('/after/' + $rootScope.data.after.liste[$stateParams.idAfter].id + '/artiste/' + $scope.artiste.id);
        $cordovaGoogleAnalytics.trackEvent('page_after_' + $rootScope.data.after.liste[$stateParams.idAfter].id, 'artiste_' + $scope.artiste.id, 'page_after_' + $rootScope.data.after.liste[$stateParams.idAfter].id + '_artiste_' + $scope.artiste.id);
      });
    }, 750);
  }

  // afficher l'icone de menu si on n'a perdu l'historique
  $scope.$on('$ionicView.beforeEnter', function(e, data) {
    $scope.showMenuIcon = !data.enableBack;
  });
})

.controller('SocialCtrl', function($scope, $http, ngFB, moment, localStorageService, $cordovaGoogleAnalytics) {
  $scope.$on('$stateChangeSuccess', function() {
    document.addEventListener("deviceready", function() {
      $cordovaGoogleAnalytics.trackView('/social');
      $cordovaGoogleAnalytics.trackEvent('navigation', 'reseaux_sociaux', 'navigation_bouton_reseaux_sociaux');
    });
  });

  var getFB = function(flush) {
    if (flush || localStorageService.keys().indexOf('facebook') == -1) {
      // get oauth token from app for subsequent graph api calls
      ngFB.api({path: '/oauth/access_token', params: {
        client_id: '155979051500295',
        client_secret: '1eb895d8481ad79a9d764f44487226f9',
        grant_type: 'client_credentials'
      }}).then(function(info) {
        // info: access_token=XXXXXXXXXXXXXXXXXXXXXXXXXXX
        // on ne veut que le code en lui-même
        var access_token = info.access_token;
        // on récupère le feed technoparade
        ngFB.api({path: '/technoparadeparis/feed', params: {access_token: access_token}}).then(function(info) {
          $scope.feedFB = info.data;
          // on parse les dates pour les afficher dans la view
          for (var i = $scope.feedFB.length - 1; i >= 0; i--)
            $scope.feedFB[i].created_time = moment($scope.feedFB[i].created_time).toDate();
            localStorageService.set('facebook', $scope.feedFB);
        }, function(error) { console.log('FB error:', error.error_description); });

      }, function(error) { console.log('FB error:', error.error_description); });
    } else {
      $scope.feedFB = localStorageService.get('facebook');
    }
  };

  var getTwitter = function(flush) {
    if (flush || localStorageService.keys().indexOf('twitter') == -1) {
      $http.get('http://technoparade.trgs.at/twitter.php').then(function(res) {
        $scope.feedTwitter = res.data;

        // on parse les dates pour les afficher dans la view
        for (var i = $scope.feedTwitter.length - 1; i >= 0; i--) {
          $scope.feedTwitter[i].created_at = moment($scope.feedTwitter[i].created_at, 'ddd MMM DD HH:mm:ss Z YYYY').toDate();
        }
        localStorageService.set('twitter', $scope.feedTwitter);
      }, function(error) { console.log('FB error:', error.error_description); });
    } else {
      $scope.feedTwitter = localStorageService.get('twitter');
    }
  };

  var getInstagram = function(flush) {
    if (flush || localStorageService.keys().indexOf('instagram') == -1) {
      var url = 'http://whateverorigin.org/get?url=' + encodeURIComponent('http://instagram.com/technoparadeparis/media') + '&callback=JSON_CALLBACK';
      $http.jsonp(url).then(function(response) {
        $scope.feedInstagram = response.data.contents.items;

        // on parse les dates pour les afficher dans la view
        for (var i = $scope.feedInstagram.length - 1; i >= 0; i--) {
          $scope.feedInstagram[i].created_time = moment.unix($scope.feedInstagram[i].created_time).toDate();
        }
        localStorageService.set('instagram', $scope.feedInstagram);
      }, function(error) { console.log('Instagram error:', error); });
    } else {
      $scope.feedInstagram = localStorageService.get('instagram');
    }
  };

  $scope.refresh = function(flush) {
    getFB(flush);
    getTwitter(flush);
    getInstagram(flush);
  };

  $scope.refresh(false);
})