 var MapService = function() {
  	this.gpxData = null;
	this.gpxLayer = null;
	this.map = null;

	this.styleFunction = function(feature, resolution) {
		  	var defaultStyle = {
	    'Point': new ol.style.Style({
	      image: new ol.style.Circle({
	        fill: new ol.style.Fill({
	          color: 'rgba(255,236,0,0.75)'
	        }),
	        radius: 5,
	        stroke: new ol.style.Stroke({
	          color: '#ffec00',
	          width: 5
	        })
	      })
	    }),
	    'LineString': new ol.style.Style({
	      stroke: new ol.style.Stroke({
	        color: '#f00',
	        width: 3
	      })
	    }),
	    'Polygon': new ol.style.Style({
	      fill: new ol.style.Fill({
	        color: 'rgba(0,255,255,0.5)'
	      }),
	      stroke: new ol.style.Stroke({
	        color: '#0ff',
	        width: 5
	      })
	    }),
	    'MultiPoint': new ol.style.Style({
	      image: new ol.style.Circle({
	        fill: new ol.style.Fill({
	          color: 'rgba(255,236,0,0.75)'
	        }),
	        radius: 5,
	        stroke: new ol.style.Stroke({
	          color: '#ffec00',
	          width: 5
	        })
	      })
	    }),
	    'MultiLineString': new ol.style.Style({
	      stroke: new ol.style.Stroke({
	        color: 'rgba(0,255,0,0.5)',
	        width: 10
	      })
	    }),
	    'MultiPolygon': new ol.style.Style({
	      fill: new ol.style.Fill({
	        color: 'rgba(0,0,255,0.5)'
	      }),
	      stroke: new ol.style.Stroke({
	        color: '#00f',
	        width: 1
	      })
	    })
	};

		var featureStyleFunction = feature.getStyleFunction();
		if (featureStyleFunction) {
			return featureStyleFunction.call(feature, resolution);
		} else {
			return defaultStyle[feature.getGeometry().getType()];
		}
	};

  	this.setGpxData = function(data) {
  		this.gpxData = data;
  		this.init();
  	}

  	this.init = function() {
  		var __this = this;
		var gpxFormat = new ol.format.GPX();
		var gpxFeatures = gpxFormat.readFeatures(this.gpxData, {
			dataProjection:'EPSG:4326',
			featureProjection:'EPSG:3857'
		});

		this.gpxLayer = new ol.layer.Vector({
			source: new ol.source.Vector({
		}),
			style: __this.styleFunction
		});

		this.gpxLayer.getSource().addFeatures(gpxFeatures);
  	}

  	this.drawMap = function(id, center) {
  		var __this = this;
		this.map = new ol.Map({
			target: id,
			layers: [
				new ol.layer.Tile({
					source: new ol.source.OSM()
				}),
				__this.gpxLayer
			],
			view: new ol.View({
				center: ol.proj.transform(center, 'EPSG:4326', 'EPSG:3857'),
				zoom: 16
			})
		});
  	}

  	this.addMarker = function(img, pos) {
  		console.log(img, pos)
  		pos = ol.proj.fromLonLat(pos);
  		element = document.createElement('img'); 
  		element.src = img;
  		element.width = "35";
		var marker = new ol.Overlay({
			position: pos,
			element: element,
			positioning: 'bottom-center',
		});
		this.map.addOverlay(marker);
		return marker;
  	}
  }